@@ -1,8 +1,17 @@
+/*
+ * gnome-print.c: Main GnomePrint API
+ *
+ * Authors:
+ *   Raph Levien (raph@acm.org)
+ *   Miguel de Icaza (miguel@kernel.org)
+ */
 #include <gtk/gtk.h>
 #include <stdio.h>
 #include <libart_lgpl/art_affine.h>
 #include <libgnomeprint/gnome-print.h>
 #include <libgnomeprint/gnome-print-ps.h>
+#include <libgnomeprint/gnome-print-pixbuf.h>
+#include <string.h>
 
 static void gnome_print_context_class_init (GnomePrintContextClass *klass);
 
@@ -93,16 +102,16 @@ gnome_print_context_init (GnomePrintContext *pc)
    However, this will do for now, and saves making extra classes.
 */
 GnomePrintContext *
-gnome_print_context_new (GnomePrinter *printer)
+gnome_print_context_new (GnomePrinter *printer, const char *paper_size)
 {
-  switch (printer->printer_type)
-    {
-    case GNOME_PRINTER_TYPE_GENERIC_PS:
-      return GNOME_PRINT_CONTEXT(gnome_print_ps_new (printer));
-    }
-  return NULL;
-}
+	if (strcmp (printer->driver, "gnome-print-ps") == 0)
+		return GNOME_PRINT_CONTEXT (gnome_print_ps_new (printer));
 
+	if (strncmp (printer->driver, "gnome-print-uni", strlen ("gnome-print-uni")) == 0)
+		return GNOME_PRINT_CONTEXT (gnome_print_pixbuf_new (printer, paper_size, 75));
+
+	return NULL;
+}
 
 static void
 gnome_print_context_finalize (GtkObject *object)
@@ -219,7 +228,7 @@ gnome_print_setfont          (GnomePrintContext *pc, GnomeFont *font)
 }
 
 int
-gnome_print_show             (GnomePrintContext *pc, char *text)
+gnome_print_show             (GnomePrintContext *pc, char const *text)
 {
   return PRINT_CLASS(pc)->show (pc, text);
 }
