@@ -9,6 +9,65 @@
 #include <libgnomeprint/gnome-printer-profile.h>
 #include <libgnomeprint/gnome-printer-dialog.h>
 
+static char *dump_file, *load_file;
+static int do_preview, test2;
+
+static struct poptOption options [] = {
+	{ "dump", 0, POPT_ARG_STRING, &dump_file },
+	{ "load", 0, POPT_ARG_STRING, &load_file },
+	{ "preview", 0, POPT_ARG_NONE, &do_preview },
+	{ "test2",   0, POPT_ARG_NONE, &test2 },
+	{ NULL }
+};
+
+static void
+print_test2_page (GnomePrintContext *pc)
+{
+	double image_matrix [6] = {30, 0, 0, 30, 0, 0};
+	char colorimg [256][256][3];
+	int x, y;
+	
+	for (y = 0; y < 256; y++)
+		for (x = 0; x < 256; x++)
+		{
+			colorimg[y][x][0] = (x + y) >> 1;
+			colorimg[y][x][1] = (x + (255 - y)) >> 1;
+			colorimg[y][x][2] = ((255 - x) + y) >> 1;
+		}
+
+	for (y = 0; y < 256; y++){
+		for (x = 0; x < 30; x++){
+			colorimg [y][x][0] = 0;
+			colorimg [y][x][1] = 0;
+			colorimg [y][x][2] = 0;
+		}
+	}
+
+	for (x = 0; x < 256; x++){
+		for (y = 0; y < 30; y++){
+			colorimg [y][x][0] = 0;
+			colorimg [y][x][1] = 0;
+			colorimg [y][x][2] = 0;
+		}
+	}
+
+	gnome_print_gsave (pc);
+	for (x = 0; x < 200; x += 100)
+		for (y = 0; y < 200; y += 100){
+			gnome_print_grestore (pc);
+			gnome_print_gsave (pc);
+
+			if (x == 100 && y == 100)
+				break;
+			
+			image_matrix [4] = x;
+			image_matrix [5] = y;			
+			gnome_print_concat (pc, image_matrix);
+
+			gnome_print_rgbimage (pc, (char *) colorimg, 256, 256, 768);
+		}
+}
+
 static void
 print_test_page (GnomePrintContext *pc)
 
@@ -54,6 +113,14 @@ print_test_page (GnomePrintContext *pc)
   gnome_print_setfont (pc, font);
   gnome_print_show (pc, "Gnome-print test page");
   gnome_print_grestore (pc);
+
+  gnome_print_setrgbcolor (pc, 1, 0, 0.5);
+  gnome_print_moveto (pc, 50, 50);
+  gnome_print_lineto (pc, 50, 90);
+  gnome_print_curveto (pc, 70, 90, 90, 70, 90, 50);
+
+  gnome_print_show (pc, "Gnome-print test page");
+  gnome_print_grestore (pc);
 #endif
 
   gnome_print_setrgbcolor (pc, 1, 0, 0.5);
@@ -117,7 +184,7 @@ print_test_page (GnomePrintContext *pc)
   gtk_object_unref (GTK_OBJECT (font));
 }
 
-void
+static void
 test_dump_metafile (char *file)
 {
 	GnomePrintMeta *meta;
@@ -139,7 +206,7 @@ test_dump_metafile (char *file)
 	fclose (f);
 }
 
-void
+static void
 test_load_metafile (char *file)
 {
 	GnomePrinter *printer;
@@ -175,7 +242,7 @@ test_load_metafile (char *file)
 	g_free (buffer);
 	
 	printer = gnome_printer_dialog_new_modal ();
-	pc = gnome_print_context_new (printer);
+	pc = gnome_print_context_new (printer, "US-Letter");
 
 	gnome_print_meta_render_from_object (pc, meta);
 	gtk_object_unref (GTK_OBJECT (meta));
@@ -183,46 +250,65 @@ test_load_metafile (char *file)
 	gtk_object_unref (GTK_OBJECT (printer));
 }
 
-void
+static void
 test_ps (void)
 {
-  GnomePrinter *printer;
-  GnomePrintContext *pc;
-  
-  printer = gnome_printer_dialog_new_modal ();
+	GnomePrinter *printer;
+	GnomePrintContext *pc;
 
-  if (printer)
-    {
-      pc = gnome_print_context_new (printer);
-      print_test_page (pc);
-      gtk_object_unref (GTK_OBJECT (printer));
-      gtk_object_unref (GTK_OBJECT (pc));
-    }
+	if (do_preview){
+		GtkWidget *toplevel, *canvas, *sw;
+
+		gtk_widget_set_default_colormap (gdk_rgb_get_cmap ());
+		gtk_widget_set_default_visual (gdk_rgb_get_visual ());
+
+		toplevel = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+		gtk_widget_set_usize (toplevel, 700, 700);
+		sw = gtk_scrolled_window_new (NULL, NULL);
+		canvas = gnome_canvas_new_aa ();
+		gtk_container_add (GTK_CONTAINER (toplevel), sw);
+		gtk_container_add (GTK_CONTAINER (sw), canvas);
+		
+		pc = gnome_print_preview_new (canvas, "US-Letter");
+
+		gtk_widget_show_all (toplevel);
+
+	} else {
+		printer = gnome_printer_dialog_new_modal ();
+	
+		if (!printer)
+			return;
+	
+		pc = gnome_print_context_new (printer, "US-Letter");
+	}
+
+	if (test2)
+		print_test2_page (pc);
+	else
+		print_test_page (pc);
+
+	if (do_preview)
+		gtk_main ();
+			
+	gtk_object_unref (GTK_OBJECT (printer));
+	gtk_object_unref (GTK_OBJECT (pc));
 }
 
-static char *dump_file, *load_file;
-
-static struct poptOption options [] = {
-	{ "dump", 0, POPT_ARG_STRING, &dump_file },
-	{ "load", 0, POPT_ARG_STRING, &load_file },
-	{ NULL }
-};
-
 int
 main (int argc, char **argv)
 {
-  gnome_init_with_popt_table ("TestPrint", VERSION, argc, argv, options, 0, NULL);
+	gnome_init_with_popt_table ("TestPrint", VERSION, argc, argv, options, 0, NULL);
+	
+	if (dump_file){
+		test_dump_metafile (dump_file);
+		return 0;
+	}
+	
+	if (load_file){
+		test_load_metafile (load_file);
+		return 0;
+	}
 
-  if (dump_file){
-	  test_dump_metafile (dump_file);
-	  return 0;
-  }
-
-  if (load_file){
-	  test_load_metafile (load_file);
-	  return 0;
-  }
-
-  test_ps ();
-  return 0;
+	test_ps ();
+	return 0;
 }
